# -*- coding: utf-8 -*-
from __future__ import absolute_import, print_function, unicode_literals

import os
import os.path
import shutil
import sys
from copy import copy
from distutils.version import LooseVersion
from tempfile import mkdtemp

import django

from app_helper import runner
from app_helper.default_settings import get_boilerplates_settings
from app_helper.main import _make_settings, _map_argv, core
from app_helper.utils import DJANGO_1_9, captured_output, temp_dir, work_in

try:
    from unittest.mock import patch
except ImportError:
    from mock import patch

try:
    import unittest2 as unittest
except ImportError:
    import unittest

DEFAULT_ARGS = {
    'shell': False,
    'test': False,
    'cms_check': False,
    'compilemessages': False,
    'makemessages': False,
    'makemigrations': False,
    'pyflakes': False,
    'authors': False,
    'server': False,
    '--xvfb': '',
    '--runner': None,
    '--runner-options': None,
    '--cms': True,
    '--failfast': False,
    '--merge': False,
    '--locale': '',
    '--boilerplate': False,
    '--dry-run': False,
    '--empty': False,
    '--native': False,
    '--persistent': False,
    '--persistent-path': None,
    '--bind': '',
    '--port': '',
    '<test-label>': '',
    '<extra-applications>': '',
    'options': '',
    '<command>': '',
}


class CommandTests(unittest.TestCase):
    application = None
    basedir = None
    pofile = None
    mofile = None
    migration_dir = None

    @classmethod
    def setUpClass(cls):
        os.environ.setdefault('DATABASE_URL', 'sqlite://localhost/:memory:')
        cls.basedir = os.path.abspath(os.path.join('app_helper', 'test_utils'))
        cls.application = 'example1'
        cls.application_2 = 'example2'
        with work_in(cls.basedir):
            with captured_output():
                cls.migration_example = os.path.abspath(
                    os.path.join(cls.application, 'data', 'django_0001_initial.py'))
                cls.migration_partial = os.path.abspath(
                    os.path.join(cls.application, 'data', 'django_0001_partial.py'))
                cls.poexample = os.path.abspath(os.path.join(cls.application, 'data', 'django.po'))
                cls.pofile = os.path.abspath(os.path.join(cls.application, 'locale', 'en', 'LC_MESSAGES', 'django.po'))
                cls.mofile = os.path.abspath(os.path.join(cls.application, 'locale', 'en', 'LC_MESSAGES', 'django.mo'))
                cls.migration_dir = os.path.abspath(os.path.join(cls.application, 'migrations'))
                cls.migration_dir_2 = os.path.abspath(os.path.join(cls.application_2, 'migrations'))
                cls.migration_file = os.path.abspath(os.path.join(cls.application, 'migrations', '0001_initial.py'))
                cls.migration_file_2 = os.path.abspath(
                    os.path.join(cls.application_2, 'migrations', '0001_initial.py'))
        try:
            import cms
        except ImportError:
            DEFAULT_ARGS['--cms'] = False

    def setUp(self):
        try:
            os.unlink(self.pofile)
        except (OSError, TypeError):
            pass
        try:
            os.unlink(self.mofile)
        except (OSError, TypeError):
            pass
        try:
            if self.migration_dir:
                shutil.rmtree(self.migration_dir)
        except (OSError, TypeError):
            pass
        try:
            if self.migration_dir_2:
                shutil.rmtree(self.migration_dir_2)
        except (OSError, TypeError):
            pass
        try:
            del sys.modules['example1.migrations']
        except KeyError as e:
            pass
        try:
            del sys.modules['example2.migrations']
        except KeyError as e:
            pass

    def tearDown(self):
        os.environ['AUTH_USER_MODEL'] = 'auth.User'
        self.setUp()

    def test_map_argv(self):
        from ..main import _map_argv

        argv_1 = [
            'helper.py',
            'example1',
            'test',
            '--cms',
            '--persistent-path=test',
            '--extra-settings==something.py',
            '--runner==something.py',
        ]
        target_1 = {
            '--bind': '127.0.0.1',
            '--boilerplate': False,
            '--cms': True,
            '--dry-run': False,
            '--empty': False,
            '--extra-settings': '',
            '--failfast': False,
            '--help': False,
            '--locale': None,
            '--merge': False,
            '--migrate': False,
            '--native': False,
            '--no-migrate': False,
            '--persistent': True,
            '--persistent-path': 'test',
            '--port': '8000',
            '--runner': '',
            '--runner-options': None,
            '--verbose': None,
            '--version': False,
            '--xvfb': False,
            '<application>': 'example1',
            '<command>': None,
            '<extra-applications>': [],
            '<test-label>': [],
            'authors': False,
            'cms_check': False,
            'compilemessages': False,
            'makemessages': False,
            'makemigrations': False,
            'options': ['helper.py', 'test', '--cms', '--persistent-path=test',
                        '--extra-settings==something.py', '--runner==something.py'],
            'pyflakes': False,
            'server': False,
            'setup': False,
            'test': True
        }
        argv_2 = [
            'helper.py',
            'example1',
            'server',
            '--cms',
            '--persistent-path=test',
            '--extra-settings==something.py',
            '--runner==something.py',
        ]
        target_2 = {
            '--bind': '127.0.0.1',
            '--boilerplate': False,
            '--cms': True,
            '--dry-run': False,
            '--empty': False,
            '--extra-settings': '',
            '--failfast': False,
            '--help': False,
            '--locale': None,
            '--merge': False,
            '--migrate': False,
            '--native': False,
            '--no-migrate': False,
            '--persistent': True,
            '--persistent-path': 'test',
            '--port': '8000',
            '--runner': '',
            '--runner-options': None,
            '--verbose': None,
            '--version': False,
            '--xvfb': False,
            '<application>': 'example1',
            '<command>': None,
            '<extra-applications>': [],
            '<test-label>': [],
            'authors': False,
            'cms_check': False,
            'compilemessages': False,
            'makemessages': False,
            'makemigrations': False,
            'options': ['helper.py', 'server', '--cms', '--persistent-path=test',
                        '--extra-settings==something.py', '--runner==something.py'],
            'pyflakes': False,
            'server': True,
            'setup': False,
            'test': False
        }

        argv_3 = [
            'helper.py',
            'example1',
            'some_command',
            '--cms',
            '--persistent-path=test',
            '--extra-settings==something.py',
            '--runner==something.py',
        ]
        target_3 = {
            '--bind': '127.0.0.1',
            '--boilerplate': False,
            '--cms': True,
            '--dry-run': False,
            '--empty': False,
            '--extra-settings': '',
            '--failfast': False,
            '--help': False,
            '--locale': None,
            '--merge': False,
            '--migrate': False,
            '--native': False,
            '--no-migrate': False,
            '--persistent': True,
            '--persistent-path': 'test',
            '--port': '8000',
            '--runner': '',
            '--runner-options': None,
            '--verbose': None,
            '--version': False,
            '--xvfb': False,
            '<application>': 'example1',
            '<command>': 'some_command',
            '<extra-applications>': [],
            '<test-label>': [],
            'authors': False,
            'cms_check': False,
            'compilemessages': False,
            'makemessages': False,
            'makemigrations': False,
            'options': ['helper.py', 'some_command', '--cms', '--persistent-path=test',
                        '--extra-settings==something.py', '--runner==something.py'],
            'pyflakes': False,
            'server': False,
            'setup': False,
            'test': False
        }

        application_module = __import__(argv_1[1])
        args = _map_argv(argv_1, application_module)
        self.assertEqual(target_1, args)

        application_module = __import__(argv_2[1])
        args = _map_argv(argv_2, application_module)
        self.assertEqual(target_2, args)

        application_module = __import__(argv_3[1])
        args = _map_argv(argv_3, application_module)
        self.assertEqual(target_3, args)

    def test_extra_settings(self):
        from django.conf import settings

        with work_in(self.basedir):
            # with captured_output() as (out, err):
            args = copy(DEFAULT_ARGS)
            with temp_dir() as STATIC_ROOT:
                with temp_dir() as MEDIA_ROOT:
                    local_settings = _make_settings(args, self.application,
                                                    settings,
                                                    STATIC_ROOT, MEDIA_ROOT)
                    # Testing that helper.py in custom project is loaded
                    self.assertEqual(local_settings.TIME_ZONE, 'Europe/Rome')

                    args['--boilerplate'] = True
                    args['--extra-settings'] = 'cms_helper_extra.py'
                    local_settings = _make_settings(args, self.application,
                                                    settings,
                                                    STATIC_ROOT, MEDIA_ROOT)
                    # Testing that helper.py in the command option is loaded
                    self.assertEqual(local_settings.TIME_ZONE, 'Europe/Paris')
                    # Existing application is kept
                    self.assertTrue('app_helper.test_data' in local_settings.INSTALLED_APPS)
                    # New ones are added both on top and in random positions
                    self.assertEqual('djangocms_admin_style', local_settings.INSTALLED_APPS[0])
                    self.assertTrue('some_app' in local_settings.INSTALLED_APPS)

                    # Ditto for middlewares
                    if DJANGO_1_9:
                        self.assertEqual('top_middleware', local_settings.MIDDLEWARE_CLASSES[0])
                        self.assertTrue('some_middleware' in local_settings.MIDDLEWARE_CLASSES)
                        self.assertTrue(
                            'django.contrib.sessions.middleware.SessionMiddleware' in local_settings.MIDDLEWARE_CLASSES)
                    else:
                        self.assertTrue(
                            'django.contrib.sessions.middleware.SessionMiddleware' in local_settings.MIDDLEWARE)
                        self.assertEqual('top_middleware', local_settings.MIDDLEWARE[0])
                        self.assertTrue('some_middleware' in local_settings.MIDDLEWARE)

                    boilerplate_settings = get_boilerplates_settings()

                    # Check the loaders
                    self.assertTrue(
                        'django.template.loaders.app_directories.Loader' in local_settings.TEMPLATES[0]['OPTIONS'][
                            'loaders'])
                    # Loaders declared in settings
                    self.assertTrue(
                        'admin_tools.template_loaders.Loader' in local_settings.TEMPLATES[0]['OPTIONS']['loaders'])
                    # Existing application is kept
                    self.assertTrue(
                        'django.template.context_processors.request' in local_settings.TEMPLATES[0]['OPTIONS'][
                            'context_processors'])
                    # New one is added
                    self.assertTrue(
                        'django.template.context_processors.debug' in local_settings.TEMPLATES[0]['OPTIONS'][
                            'context_processors'])
                    # Check template dirs
                    self.assertTrue('some/dir' in local_settings.TEMPLATES[0]['DIRS'])
                    # Check for aldryn boilerplates
                    for name, value in boilerplate_settings.items():
                        if not name.startswith('TEMPLATE'):
                            if type(value) in (list, tuple):
                                self.assertTrue(set(getattr(local_settings, name)).intersection(set(value)))
                            elif name == 'ALDRYN_BOILERPLATE_NAME':
                                self.assertEqual(getattr(local_settings, name), 'legacy')
                            else:
                                self.assertTrue(value in getattr(local_settings, name))
                        elif name == 'TEMPLATE_CONTEXT_PROCESSORS':
                            self.assertTrue(
                                set(local_settings.TEMPLATES[0]['OPTIONS']['context_processors']).intersection(
                                    set(value)))
                        elif name == 'TEMPLATE_LOADERS':
                            self.assertTrue(
                                set(local_settings.TEMPLATES[0]['OPTIONS']['loaders']).intersection(set(value)))

    @patch('app_helper.main.autoreload')
    def test_server(self, mocked_command):
        with work_in(self.basedir):
            with captured_output() as (out, err):
                args = copy(DEFAULT_ARGS)
                args['server'] = True
                core(args, self.application)
            self.assertTrue(
                'A admin user (username: admin, password: admin) has been created.' in
                out.getvalue()
            )

    def test_makemigrations(self):
        with work_in(self.basedir):
            with captured_output() as (out, err):
                args = copy(DEFAULT_ARGS)
                args['makemigrations'] = True
                args['<extra-applications>'] = ['example2']
                core(args, self.application)
            self.assertTrue(os.path.exists(self.migration_file))
            self.assertTrue(os.path.exists(self.migration_file_2))
        self.assertTrue('Create model ExampleModel1' in out.getvalue())
        self.assertTrue('Create model ExampleModel2' in out.getvalue())

    def skip_test_makemigrations_update(self):
        os.makedirs(self.migration_dir)
        open(os.path.join(self.migration_dir, '__init__.py'), 'w')
        shutil.copy(self.migration_partial, self.migration_file)
        with work_in(self.basedir):
            with captured_output() as (out, err):
                args = copy(DEFAULT_ARGS)
                args['makemigrations'] = True
                core(args, self.application)
        self.assertTrue('Migrations for \'example1\':' in out.getvalue())

    def test_makemigrations_empty(self):
        with work_in(self.basedir):
            with captured_output() as (out, err):
                os.makedirs(self.migration_dir)
                open(os.path.join(self.migration_dir, '__init__.py'), 'w')
                shutil.copy(self.migration_example, self.migration_file)
                args = copy(DEFAULT_ARGS)
                args['makemigrations'] = True
                args['--empty'] = True
                core(args, self.application)
        self.assertTrue('Migrations for \'example1\':' in out.getvalue())

    def test_makemigrations_merge(self):
        with work_in(self.basedir):
            with captured_output() as (out, err):
                args = copy(DEFAULT_ARGS)
                args['makemigrations'] = True
                args['--merge'] = True
                core(args, self.application)
                self.assertTrue('No conflicts detected to merge' in out.getvalue())

    def test_makemessages(self):
        with work_in(self.basedir):
            with captured_output() as (out, err):
                args = copy(DEFAULT_ARGS)
                args['makemessages'] = True
                core(args, self.application)
                self.assertTrue(os.path.exists(self.pofile))

    def test_compilemessages(self):
        with work_in(self.basedir):
            shutil.copy(self.poexample, self.pofile)
            args = copy(DEFAULT_ARGS)
            args['compilemessages'] = True
            core(args, self.application)
            self.assertTrue(os.path.exists(self.mofile))

    def test_cms_check(self):
        try:
            import cms
        except ImportError:
            raise unittest.SkipTest('django CMS not available, skipping test')
        with work_in(self.basedir):
            with captured_output() as (out, err):
                shutil.copy(self.poexample, self.pofile)
                args = copy(DEFAULT_ARGS)
                args['cms_check'] = True
                args['--extra-settings'] = 'helper.py'
                args['--migrate'] = False
                core(args, self.application)
            self.assertTrue('Installation okay' in out.getvalue())
            self.assertFalse('[WARNING]' in out.getvalue())
            self.assertFalse('[ERROR]' in out.getvalue())

    @unittest.skipIf(LooseVersion(django.get_version()) >= LooseVersion('1.9'),
                     reason='Test for Django up until 1.8')
    def test_cms_check_nocms(self):
        try:
            import cms
            raise unittest.SkipTest('django CMS available, skipping test')
        except ImportError:
            pass
        with work_in(self.basedir):
            with captured_output() as (out, err):
                shutil.copy(self.poexample, self.pofile)
                args = copy(DEFAULT_ARGS)
                args['cms_check'] = True
                args['--extra-settings'] = 'helper.py'
                args['--migrate'] = False
                core(args, self.application)
            self.assertTrue('cms_check available only if django CMS is installed' in out.getvalue())

    @unittest.skipIf(LooseVersion(django.get_version()) < LooseVersion('1.9'),
                     reason='Test for Django 1.9+')
    def test_cms_check_nocms_19(self):
        try:
            import cms
            raise unittest.SkipTest('django CMS available, skipping test')
        except ImportError:
            pass
        with work_in(self.basedir):
            with self.assertRaises(ImportError):
                shutil.copy(self.poexample, self.pofile)
                args = copy(DEFAULT_ARGS)
                args['cms_check'] = True
                args['--extra-settings'] = 'helper.py'
                args['--migrate'] = False
                core(args, self.application)

    @unittest.skipIf(LooseVersion(django.get_version()) < LooseVersion('1.7'),
                     reason='check command available in Django 1.7+ only')
    def test_any_command_check(self):
        with work_in(self.basedir):
            with captured_output() as (out, err):
                args = copy(DEFAULT_ARGS)
                args['<command>'] = 'check'
                args['options'] = ['helper', 'check', '--extra-settings=cms_helper_extra.py']
                core(args, self.application)
        self.assertTrue('no issues' in out.getvalue())

    def test_any_command_compilemessages(self):
        with work_in(os.path.join(self.basedir, self.application)):
            with captured_output() as (out, err):
                shutil.copy(self.poexample, self.pofile)
                args = copy(DEFAULT_ARGS)
                args['<command>'] = 'compilemessages'
                args['options'] = 'helper compilemessages --cms -len --verbosity=2'.split(' ')
                core(args, self.application)
                self.assertTrue(os.path.exists(self.mofile))

    def test_any_command_migrations(self):
        with work_in(self.basedir):
            with captured_output() as (out, err):
                args = copy(DEFAULT_ARGS)
                args['<command>'] = 'makemigrations'
                args['options'] = 'helper makemigrations example2 --verbosity=2'.split(' ')
                core(args, self.application)
            self.assertFalse('Create model ExampleModel1' in out.getvalue())
            self.assertFalse(os.path.exists(self.migration_file))
            self.assertTrue('Create model ExampleModel2' in out.getvalue())
            self.assertTrue(os.path.exists(self.migration_file_2))

    def test_pyflakes(self):
        try:
            import cms
        except ImportError:
            raise unittest.SkipTest('django CMS not available, skipping test')
        with work_in(self.basedir):
            with captured_output() as (out, err):
                args = copy(DEFAULT_ARGS)
                args['pyflakes'] = True
                core(args, self.application)
        self.assertFalse(os.path.exists(args['STATIC_ROOT']))
        self.assertFalse(os.path.exists(args['MEDIA_ROOT']))

    def test_pyflakes_nocms(self):
        try:
            import cms
            raise unittest.SkipTest('django CMS available, skipping test')
        except ImportError:
            pass
        with work_in(self.basedir):
            with captured_output() as (out, err):
                args = copy(DEFAULT_ARGS)
                args['pyflakes'] = True
                core(args, self.application)
            self.assertTrue(
                'Static analysis available only if django CMS and pyflakes are installed' in out.getvalue()
            )
        self.assertFalse(os.path.exists(args['STATIC_ROOT']))
        self.assertFalse(os.path.exists(args['MEDIA_ROOT']))

    def test_testrun(self):
        try:
            import cms
        except ImportError:
            raise unittest.SkipTest('django CMS not available, skipping test')
        path = os.path.join(self.basedir, 'data')
        with work_in(self.basedir):
            with captured_output() as (out, err):
                with self.assertRaises(SystemExit) as exit:
                    try:
                        from django.test.utils import _TestState
                        del _TestState.saved_data
                    except (ImportError, AttributeError):
                        pass
                    args = copy(DEFAULT_ARGS)
                    args['test'] = True
                    args['--persistent'] = True
                    args['--runner'] = 'runners.CapturedOutputRunner'
                    core(args, self.application)
        self.assertTrue('Ran 14 tests in' in err.getvalue())
        self.assertEqual(exit.exception.code, 0)
        self.assertTrue(args['STATIC_ROOT'].startswith(path))
        self.assertTrue(args['MEDIA_ROOT'].startswith(path))
        self.assertTrue(os.path.exists(args['STATIC_ROOT']))
        self.assertTrue(os.path.exists(args['MEDIA_ROOT']))

    def test_testrun_persistent_path(self):
        try:
            import cms
        except ImportError:
            raise unittest.SkipTest('django CMS not available, skipping test')
        path = mkdtemp()
        with work_in(self.basedir):
            with captured_output() as (out, err):
                with self.assertRaises(SystemExit) as exit:
                    try:
                        from django.test.utils import _TestState
                        del _TestState.saved_data
                    except (ImportError, AttributeError):
                        pass
                    args = copy(DEFAULT_ARGS)
                    args['test'] = True
                    args['--persistent'] = True
                    args['--persistent-path'] = path
                    args['--runner'] = 'runners.CapturedOutputRunner'
                    core(args, self.application)
        self.assertTrue('Ran 14 tests in' in err.getvalue())
        self.assertEqual(exit.exception.code, 0)
        self.assertTrue(args['STATIC_ROOT'].startswith(path))
        self.assertTrue(args['MEDIA_ROOT'].startswith(path))
        self.assertTrue(os.path.exists(args['STATIC_ROOT']))
        self.assertTrue(os.path.exists(args['MEDIA_ROOT']))

    def test_runner_wrong(self):
        try:
            import cms
        except ImportError:
            raise unittest.SkipTest('django CMS not available, skipping test')
        with work_in(self.basedir):
            if sys.version_info < (3, 5):
                exception = ImportError
            else:
                exception = SystemExit
            with captured_output() as (out, err):
                with self.assertRaises(exception) as exit:
                    args = list()
                    args.append('helper')
                    args.append('test')
                    args.append('example1')
                    args.append('--runner=runners.CapturedOutputRunner')
                    args.append('whatever')
                    runner.cms('example1', args)
        if sys.version_info >= (3, 5):
            self.assertEqual(exit.exception.code, 1)

    def test_runner(self):
        try:
            import cms
        except ImportError:
            raise unittest.SkipTest('django CMS not available, skipping test')
        from app_helper.test_utils.runners import CapturedOutputRunner
        with patch('django.test.runner.DiscoverRunner', CapturedOutputRunner):
            with work_in(self.basedir):
                with captured_output() as (out, err):
                    with self.assertRaises(SystemExit) as exit:
                        args = list()
                        args.append('helper')
                        args.append('test')
                        args.append('--extra-settings=helper.py')
                        args.append('example1')
                        runner.cms('example1', args)
        self.assertTrue('visible string' in out.getvalue())
        self.assertFalse('hidden string' in out.getvalue())
        self.assertFalse('hidden string' in err.getvalue())
        self.assertTrue('Ran 14 tests in' in err.getvalue())
        self.assertEqual(exit.exception.code, 0)

    def test_runner_compat(self):
        try:
            import cms
        except ImportError:
            raise unittest.SkipTest('django CMS not available, skipping test')
        from app_helper.test_utils.runners import CapturedOutputRunner
        with patch('django.test.runner.DiscoverRunner', CapturedOutputRunner):
            with work_in(self.basedir):
                with captured_output() as (out, err):
                    with self.assertRaises(SystemExit) as exit:
                        args = list()
                        args.append('cms_helper')
                        args.append('test')
                        args.append('example1')
                        runner.cms('example1', args)
        self.assertTrue('visible string' in out.getvalue())
        self.assertFalse('hidden string' in out.getvalue())
        self.assertFalse('hidden string' in err.getvalue())
        self.assertTrue('Ran 14 tests in' in err.getvalue())
        self.assertEqual(exit.exception.code, 0)

    def test_runner_cms_exception(self):
        try:
            import cms
            raise unittest.SkipTest('django CMS available, skipping test')
        except ImportError:
            pass
        from app_helper.test_utils.runners import CapturedOutputRunner
        with patch('django.test.runner.DiscoverRunner', CapturedOutputRunner):
            with work_in(self.basedir):
                with captured_output() as (out, err):
                    with self.assertRaises(ImportError) as exit:
                        args = list()
                        args.append('helper')
                        runner.cms('example1', args)

    def test_runner_cms_argv(self):
        try:
            import cms
        except ImportError:
            raise unittest.SkipTest('django CMS not available, skipping test')

        def fake_runner(argv):
            return argv

        from app_helper.test_utils.runners import CapturedOutputRunner
        with patch('django.test.runner.DiscoverRunner', CapturedOutputRunner):
            with work_in(self.basedir):
                with captured_output() as (out, err):
                    args = list()
                    args.append('helper')
                    with patch('app_helper.runner.runner', fake_runner):
                        data = runner.cms('example1', args)
                    self.assertEqual(data, ['helper', 'example1', 'test', '--cms'])

    def test_runner_argv(self):
        def fake_runner(argv):
            return argv

        from app_helper.test_utils.runners import CapturedOutputRunner
        with patch('django.test.runner.DiscoverRunner', CapturedOutputRunner):
            with work_in(self.basedir):
                with captured_output() as (out, err):
                    args = list()
                    args.append('helper')
                    with patch('app_helper.runner.runner', fake_runner):
                        data = runner.run('example1', args)
                    self.assertEqual(data, ['helper', 'example1', 'test'])

    def test_setup_cms(self):
        try:
            import cms
        except ImportError:
            raise unittest.SkipTest('django CMS not available, skipping test')
        with work_in(self.basedir):
            with captured_output() as (out, err):
                from app_helper.test_utils import helper
                settings = runner.setup(
                    'example1', helper, use_cms=True, extra_args=['--boilerplate']
                )
        self.assertTrue('example2' in settings.INSTALLED_APPS)
        self.assertTrue('aldryn_boilerplates' in settings.INSTALLED_APPS)
        self.assertTrue('cms' in settings.INSTALLED_APPS)

    def test_setup_custom_user(self):
        os.environ['AUTH_USER_MODEL'] = 'custom_user.CustomUser'
        try:
            import cms
        except ImportError:
            raise unittest.SkipTest('django CMS not available, skipping test')
        with work_in(self.basedir):
            with captured_output() as (out, err):
                from app_helper.test_utils import cms_helper_custom
                settings = runner.setup(
                    'example1', cms_helper_custom, use_cms=True, extra_args=['--boilerplate']
                )
        self.assertTrue('example2' in settings.INSTALLED_APPS)
        self.assertTrue('custom_user' in settings.INSTALLED_APPS)
        self.assertTrue('aldryn_boilerplates' in settings.INSTALLED_APPS)
        self.assertTrue('cms' in settings.INSTALLED_APPS)
        del os.environ['AUTH_USER_MODEL']

    def test_setup_nocms(self):
        with work_in(self.basedir):
            with captured_output() as (out, err):
                from app_helper.test_utils import helper
                settings = runner.setup('example1', helper, extra_args=[])
        self.assertTrue('example2' in settings.INSTALLED_APPS)
        self.assertFalse('aldryn_boilerplates' in settings.INSTALLED_APPS)
        self.assertFalse('cms' in settings.INSTALLED_APPS)

    def test_testrun_nocms(self):
        with work_in(self.basedir):
            with captured_output() as (out, err):
                with self.assertRaises(SystemExit) as exit:
                    try:
                        from django.test.utils import _TestState
                        del _TestState.saved_data
                    except (ImportError, AttributeError):
                        pass
                    args = copy(DEFAULT_ARGS)
                    args['test'] = True
                    args['--cms'] = False
                    args['--runner'] = 'runners.CapturedOutputRunner'
                    core(args, self.application)
        self.assertTrue('Ran 14 tests in' in err.getvalue())
        self.assertEqual(exit.exception.code, 0)

    def test_runner_nocms(self):
        with work_in(self.basedir):
            with captured_output() as (out, err):
                with self.assertRaises(SystemExit) as exit:
                    args = list()
                    args.append('helper')
                    args.append('example1')
                    args.append('test')
                    args.append('--extra-settings=helper.py')
                    args.append('--runner=runners.CapturedOutputRunner')
                    runner.run('example1', args)
        self.assertTrue('Ran 14 tests in' in err.getvalue())
        self.assertEqual(exit.exception.code, 0)

    def test_testrun_native(self):
        try:
            import cms
        except ImportError:
            raise unittest.SkipTest('django CMS not available, skipping test')
        with work_in(self.basedir):
            with captured_output() as (out, err):
                try:
                    from django.test.utils import _TestState
                    del _TestState.saved_data
                except (ImportError, AttributeError):
                    pass
                args = copy(DEFAULT_ARGS)
                args['<command>'] = 'test'
                args['--cms'] = False
                args['--native'] = True
                args['--extra-settings'] = 'cms_helper_extra_runner.py'
                core(args, self.application)
        self.assertTrue('Ran 14 tests in' in err.getvalue())

    def test_authors(self):
        with work_in(self.basedir):
            with captured_output() as (out, err):
                args = copy(DEFAULT_ARGS)
                args['authors'] = True
                core(args, self.application)
        self.assertTrue('Generating AUTHORS' in out.getvalue())
        self.assertTrue('* Iacopo Spalletti' in out.getvalue())
        self.assertTrue('Authors (' in out.getvalue())

    def test_urls(self):
        try:
            import cms
        except ImportError:
            raise unittest.SkipTest('django CMS not available, skipping test')
        try:
            from django.urls import reverse
        except ImportError:
            from django.core.urlresolvers import reverse
        with work_in(self.basedir):
            with captured_output() as (out, err):
                shutil.copy(self.poexample, self.pofile)
                args = copy(DEFAULT_ARGS)
                args['makemessages'] = True
                core(args, self.application)
                self.assertTrue(reverse('pages-root'))

    def test_urls_nocms(self):
        try:
            from django.urls import reverse, NoReverseMatch
        except ImportError:
            from django.core.urlresolvers import reverse, NoReverseMatch
        with work_in(self.basedir):
            with captured_output() as (out, err):
                shutil.copy(self.poexample, self.pofile)
                args = copy(DEFAULT_ARGS)
                args['makemessages'] = True
                args['--cms'] = False
                core(args, self.application)
                with self.assertRaises(NoReverseMatch):
                    reverse('pages-root')



# -*- coding: utf-8 -*-
try:
    import djangocms_text_ckeditor  # NOQA
    text_plugin = ['djangocms_text_ckeditor']
except ImportError:
    text_plugin = []

HELPER_SETTINGS = {
    'TIME_ZONE': 'Europe/Paris',
    'INSTALLED_APPS': [
        'some_app',
    ] + text_plugin,
    'TEMPLATE_CONTEXT_PROCESSORS': [
        'django.core.context_processors.debug'
    ],
    'TEMPLATE_LOADERS': [
        'admin_tools.template_loaders.Loader',
    ],
    'TOP_MIDDLEWARE_CLASSES': [
        'top_middleware',
    ],
    'MIDDLEWARE_CLASSES': [
        'some_middleware',
    ],
    'TOP_INSTALLED_APPS': [
        'djangocms_admin_style'
    ],
    'ALDRYN_BOILERPLATE_NAME': 'legacy',
    'TEMPLATE_DIRS': [
        'some/dir'
    ],
}



# -*- coding: utf-8 -*-
from app_helper.test_utils.helper import HELPER_SETTINGS

HELPER_SETTINGS['TEMPLATE_CONTEXT_PROCESSORS'] = ['django.core.context_processors.debug']
HELPER_SETTINGS['TEST_RUNNER'] = 'runners.CapturedOutputRunner'
HELPER_SETTINGS['INSTALLED_APPS'].remove('djangocms_text_ckeditor')



# -*- coding: utf-8 -*-
from __future__ import absolute_import, print_function, unicode_literals

import contextlib
import os
import shutil
import sys
from distutils.version import LooseVersion
from tempfile import mkdtemp

import django
from django.core.management import call_command
from django.urls import clear_url_caches
from django.utils import six
from django.utils.functional import empty
from django.utils.six import StringIO
from django.utils.six.moves import reload_module

from . import HELPER_FILE

try:
    from unittest.mock import patch
except ImportError:
    from mock import patch


try:
    import cms  # NOQA
    CMS = True
    CMS_35 = LooseVersion('3.5') <= LooseVersion(cms.__version__) < LooseVersion('3.6')
    CMS_34 = LooseVersion('3.4') <= LooseVersion(cms.__version__) < LooseVersion('3.5')
    CMS_33 = LooseVersion('3.3') <= LooseVersion(cms.__version__) < LooseVersion('3.4')
    CMS_32 = LooseVersion('3.2') <= LooseVersion(cms.__version__) < LooseVersion('3.3')
    CMS_31 = LooseVersion('3.1') <= LooseVersion(cms.__version__) < LooseVersion('3.2')
    CMS_30 = LooseVersion('3.0') <= LooseVersion(cms.__version__) < LooseVersion('3.1')
except ImportError:  # pragma: no cover
    CMS = False
    CMS_35 = False
    CMS_34 = False
    CMS_33 = False
    CMS_32 = False
    CMS_31 = False
    CMS_30 = False

DJANGO_1_4 = LooseVersion(django.get_version()) < LooseVersion('1.5')
DJANGO_1_5 = LooseVersion(django.get_version()) < LooseVersion('1.6')
DJANGO_1_6 = LooseVersion(django.get_version()) < LooseVersion('1.7')
DJANGO_1_7 = LooseVersion(django.get_version()) < LooseVersion('1.8')
DJANGO_1_8 = LooseVersion(django.get_version()) < LooseVersion('1.9')
DJANGO_1_9 = LooseVersion(django.get_version()) < LooseVersion('1.10')
DJANGO_1_10 = LooseVersion(django.get_version()) < LooseVersion('1.11')
DJANGO_1_11 = LooseVersion(django.get_version()) < LooseVersion('2.0')
DJANGO_2_0 = LooseVersion(django.get_version()) < LooseVersion('2.1')
DJANGO_2_1 = LooseVersion(django.get_version()) < LooseVersion('2.2')
DJANGO_2_2 = LooseVersion(django.get_version()) < LooseVersion('3.0')


def load_from_file(module_path):
    """
    Load a python module from its absolute filesystem path

    Borrowed from django-cms
    """
    from imp import load_module, PY_SOURCE

    imported = None
    if module_path:
        with open(module_path, 'r') as openfile:
            imported = load_module('mod', openfile, module_path, ('imported', 'r', PY_SOURCE))
    return imported


@contextlib.contextmanager
def work_in(dirname=None):
    """
    Context manager version of os.chdir. When exited, returns to the working
    directory prior to entering.

    Grabbed from cookiecutter, thanks audreyr!
    """
    curdir = os.getcwd()
    try:
        if dirname is not None:
            if dirname not in sys.path:
                sys.path.insert(0, dirname)
            os.chdir(dirname)
        yield
    finally:
        os.chdir(curdir)


@contextlib.contextmanager
def captured_output():
    with patch('sys.stdout', new_callable=StringIO) as out:
        with patch('sys.stderr', new_callable=StringIO) as err:
            yield out, err


# Borrowed from django CMS codebase
@contextlib.contextmanager
def temp_dir(suffix='', container='/dev/shm/'):
    name = make_temp_dir(suffix, container)
    yield name
    shutil.rmtree(name)


def make_temp_dir(suffix='', container='/dev/shm/'):
    if os.path.exists(container):
        return mkdtemp(suffix=suffix, dir=container)
    return mkdtemp(suffix=suffix)


@contextlib.contextmanager
def persistent_dir(suffix, container='data'):
    name = os.path.abspath(os.path.join(container, suffix))
    if not os.path.exists(name):
        os.makedirs(name)
    yield name


class DisableMigrations(object):

    def __contains__(self, item):
        return True

    def __getitem__(self, item):
        return None


def _reset_django(settings):
    """
    Hackish way to reset the django instance settings and AppConfig
    :param settings: django settings module
    """
    if settings._wrapped != empty:
        clear_url_caches()
        from django.apps import apps
        apps.clear_cache()
        settings._wrapped = empty
        clear_url_caches()


def extend_settings(settings, extra_settings, key, insertion_point):
    for item in extra_settings[key]:
        if item not in settings[key]:
            settings[key].insert(settings[key].index(insertion_point), item)
    return settings


def _make_settings(args, application, settings, STATIC_ROOT, MEDIA_ROOT):
    """
    Setup the Django settings
    :param args: docopt arguments
    :param default_settings: default Django settings
    :param settings: Django settings module
    :param STATIC_ROOT: static root directory
    :param MEDIA_ROOT: media root directory
    :return:
    """
    import dj_database_url
    from .default_settings import get_default_settings, get_boilerplates_settings

    try:
        extra_settings_file = args.get('--extra-settings')
        if not extra_settings_file:
            extra_settings_file = HELPER_FILE
        if extra_settings_file[-3:] != '.py':
            filename, __ = os.path.splitext(extra_settings_file)
            extra_settings_file = '{0}.py'.format(filename)
        extra_settings = load_from_file(extra_settings_file).HELPER_SETTINGS
    except (IOError, AttributeError):
        extra_settings = None
    default_name = ':memory:' if args['test'] else 'local.sqlite'
    db_url = os.environ.get('DATABASE_URL', 'sqlite://localhost/%s' % default_name)
    configs = {
        'DATABASES': {'default': dj_database_url.parse(db_url)},
        'STATIC_ROOT': STATIC_ROOT,
        'MEDIA_ROOT': MEDIA_ROOT,
        'USE_TZ': True,
        'USE_CMS': args['--cms'],
        'BASE_APPLICATION': application
    }

    if configs['USE_CMS'] or getattr(extra_settings, 'USE_CMS', False):
        CMS_APPS = [
            'cms',
            'menus',
            'sekizai',
        ]
        CMS_APP_STYLE = [
            'djangocms_admin_style'
        ]
        CMS_PROCESSORS = [
            'cms.context_processors.cms_settings',
            'sekizai.context_processors.sekizai',
        ]
        CMS_MIDDLEWARE = [
            'cms.middleware.language.LanguageCookieMiddleware',
            'cms.middleware.user.CurrentUserMiddleware',
            'cms.middleware.page.CurrentPageMiddleware',
            'cms.middleware.toolbar.ToolbarMiddleware',
        ]
        if not CMS_31 and args['server']:
            CMS_MIDDLEWARE.append('cms.middleware.utils.ApphookReloadMiddleware')
        URLCONF = 'app_helper.urls'
    else:
        CMS_APPS = []
        CMS_APP_STYLE = []
        CMS_MIDDLEWARE = []
        CMS_PROCESSORS = []
        URLCONF = 'app_helper.urls'

    default_settings = get_default_settings(
        CMS_APPS, CMS_PROCESSORS, CMS_MIDDLEWARE, CMS_APP_STYLE, URLCONF, application
    )
    migrate = args.get('--migrate') or not args.get('--no-migrate')
    default_settings.update(configs)

    if extra_settings:
        apps = extra_settings.pop('INSTALLED_APPS', [])
        apps_top = extra_settings.pop('TOP_INSTALLED_APPS', [])
        template_processors = extra_settings.pop('TEMPLATE_CONTEXT_PROCESSORS', [])
        template_loaders = extra_settings.pop('TEMPLATE_LOADERS', [])
        template_dirs = extra_settings.pop('TEMPLATE_DIRS', [])
        middleware = extra_settings.pop('MIDDLEWARE_CLASSES', [])
        middleware_top = extra_settings.pop('TOP_MIDDLEWARE_CLASSES', [])
        default_settings.update(extra_settings)
        for app in apps_top:
            default_settings['INSTALLED_APPS'].insert(0, app)
        default_settings['INSTALLED_APPS'].extend(apps)
        default_settings['TEMPLATE_CONTEXT_PROCESSORS'].extend(template_processors)
        default_settings['TEMPLATE_LOADERS'].extend(template_loaders)
        if 'TEMPLATE_DIRS' not in default_settings:
            default_settings['TEMPLATE_DIRS'] = []
        default_settings['TEMPLATE_DIRS'].extend(template_dirs)
        default_settings['MIDDLEWARE_CLASSES'].extend(middleware)
        for middleware in middleware_top:
            default_settings['MIDDLEWARE_CLASSES'].insert(0, middleware)

    if 'cms' in default_settings['INSTALLED_APPS']:
        if 'treebeard' not in default_settings['INSTALLED_APPS']:
            default_settings['INSTALLED_APPS'].append('treebeard')
    if ('filer' in default_settings['INSTALLED_APPS'] and
            'mptt' not in default_settings['INSTALLED_APPS']):
        default_settings['INSTALLED_APPS'].append('mptt')
    if ('filer' in default_settings['INSTALLED_APPS'] and
            'easy_thumbnails' not in default_settings['INSTALLED_APPS']):
        default_settings['INSTALLED_APPS'].append('easy_thumbnails')

    if args['--boilerplate']:
        boilerplate_settings = get_boilerplates_settings()

        # Do not override helper settings with defaults.
        if 'ALDRYN_BOILERPLATE_NAME' in default_settings.keys():
            del boilerplate_settings['ALDRYN_BOILERPLATE_NAME']

        default_settings = extend_settings(
            default_settings, boilerplate_settings, 'STATICFILES_FINDERS',
            'django.contrib.staticfiles.finders.AppDirectoriesFinder'
        )
        del boilerplate_settings['STATICFILES_FINDERS']

        default_settings = extend_settings(
            default_settings, boilerplate_settings, 'TEMPLATE_LOADERS',
            'django.template.loaders.app_directories.Loader'
        )
        del boilerplate_settings['TEMPLATE_LOADERS']

        for setting in ('INSTALLED_APPS', 'TEMPLATE_CONTEXT_PROCESSORS'):
            default_settings[setting].extend(boilerplate_settings[setting])
            del boilerplate_settings[setting]
        default_settings.update(boilerplate_settings)

    default_settings['TEMPLATES'] = [
        {'NAME': 'django',
         'BACKEND': 'django.template.backends.django.DjangoTemplates',
         'OPTIONS': {
             'context_processors': [
                 template_processor.replace('django.core', 'django.template')
                 for template_processor in default_settings.pop('TEMPLATE_CONTEXT_PROCESSORS')
             ],
             'loaders': default_settings.pop('TEMPLATE_LOADERS')
         }}
    ]
    if 'TEMPLATE_DIRS' in default_settings:
        default_settings['TEMPLATES'][0]['DIRS'] = default_settings.pop('TEMPLATE_DIRS')

    # Support for custom user models
    if 'AUTH_USER_MODEL' in os.environ:
        custom_user_app = os.environ['AUTH_USER_MODEL'].rpartition('.')[0]
        custom_user_model = '.'.join(os.environ['AUTH_USER_MODEL'].split('.')[-2:])
        if 'cms' in default_settings['INSTALLED_APPS']:
            default_settings['INSTALLED_APPS'].insert(
                default_settings['INSTALLED_APPS'].index('cms'),
                custom_user_app
            )
        else:
            default_settings['INSTALLED_APPS'].insert(
                default_settings['INSTALLED_APPS'].index('django.contrib.auth') + 1,
                custom_user_app
            )
        default_settings['AUTH_USER_MODEL'] = custom_user_model

    if args['test']:
        default_settings['SESSION_ENGINE'] = 'django.contrib.sessions.backends.cache'
    if application not in default_settings['INSTALLED_APPS']:
        default_settings['INSTALLED_APPS'].append(application)

    if not migrate:
        default_settings['MIGRATION_MODULES'] = DisableMigrations()

    if 'MIDDLEWARE' not in default_settings:
        default_settings['MIDDLEWARE'] = default_settings['MIDDLEWARE_CLASSES']
        del default_settings['MIDDLEWARE_CLASSES']

    _reset_django(settings)
    settings.configure(**default_settings)
    django.setup()
    reload_urls(settings, cms_apps=False)
    return settings


def reload_urls(settings, urlconf=None, cms_apps=True):
    if 'cms.urls' in sys.modules:
        reload_module(sys.modules['cms.urls'])
    if urlconf is None:
        urlconf = settings.ROOT_URLCONF
    if urlconf in sys.modules:
        reload_module(sys.modules[urlconf])
    clear_url_caches()
    if cms_apps:
        from cms.appresolver import clear_app_resolvers, get_app_patterns
        clear_app_resolvers()
        get_app_patterns()


def _create_db(migrate_cmd=False):
    call_command('migrate')


def get_user_model():
    from django.contrib.auth import get_user_model
    return get_user_model()


def create_user(username, email, password, is_staff=False, is_superuser=False,
                base_cms_permissions=False, permissions=None):
    from django.contrib.auth.models import Permission
    User = get_user_model()

    try:
        if User.USERNAME_FIELD == 'email':
            user = User.objects.get(**{User.USERNAME_FIELD: email})
        else:
            user = User.objects.get(**{User.USERNAME_FIELD: username})
    except User.DoesNotExist:
        user = User()

    if User.USERNAME_FIELD != 'email':
        setattr(user, User.USERNAME_FIELD, username)

    try:
        user.email = email
    except AttributeError:
        pass
    user.set_password(password)
    if is_superuser:
        user.is_superuser = True
    if is_superuser or is_staff:
        user.is_staff = True
    user.is_active = True
    user.save()
    if user.is_staff and not is_superuser and base_cms_permissions:
        user.user_permissions.add(Permission.objects.get(codename='add_text'))
        user.user_permissions.add(Permission.objects.get(codename='delete_text'))
        user.user_permissions.add(Permission.objects.get(codename='change_text'))
        user.user_permissions.add(Permission.objects.get(codename='publish_page'))

        user.user_permissions.add(Permission.objects.get(codename='add_page'))
        user.user_permissions.add(Permission.objects.get(codename='change_page'))
        user.user_permissions.add(Permission.objects.get(codename='delete_page'))
    if is_staff and not is_superuser and permissions:
        for permission in permissions:
            user.user_permissions.add(Permission.objects.get(codename=permission))
    return user


def get_user_model_labels():
    User = get_user_model()

    user_orm_label = '%s.%s' % (User._meta.app_label, User._meta.object_name)
    user_model_label = '%s.%s' % (User._meta.app_label, User._meta.model_name)
    return user_orm_label, user_model_label


class UserLoginContext(object):
    def __init__(self, testcase, user, password=None):
        self.testcase = testcase
        self.user = user
        if password is None:
            password = getattr(user, get_user_model().USERNAME_FIELD)
        self.password = password

    def __enter__(self):
        loginok = self.testcase.client.login(username=getattr(self.user,
                                                              get_user_model().USERNAME_FIELD),
                                             password=self.password)
        self.testcase.assertTrue(loginok)
        self.testcase._login_context = self

    def __exit__(self, exc, value, tb):
        self.testcase._login_context = None
        self.testcase.client.logout()


def ensure_unicoded_and_unique(args_list, application):
    """
    Iterate over args_list, make it unicode if needed and ensure that there
    are no duplicates.
    Returns list of unicoded arguments in the same order.
    """
    unicoded_args = []
    for argument in args_list:
        argument = (six.u(argument)
                    if not isinstance(argument, six.text_type) else argument)
        if argument not in unicoded_args or argument == application:
            unicoded_args.append(argument)
    return unicoded_args



# -*- coding: utf-8 -*-
from __future__ import absolute_import, print_function, unicode_literals

import setuptools

from app_helper import __version__

test_requirements = []
requirements = [
    'docopt',
    'dj-database-url',
]


setuptools.setup(
    name='django-app-helper',
    version=__version__,
    url='https://github.com/nephila/django-app-helper',
    author='Iacopo Spalletti',
    author_email='i.spalletti@nephila.it',
    description='Helper for django applications development',
    long_description=open('README.rst').read(),
    license='GPLv2+',
    packages=setuptools.find_packages(),
    include_package_data=True,
    install_requires=requirements,
    extras_require={
        ':python_version<"3.3"': ['mock'],
        'pyflakes': ['pyflakes<2.1', 'django-cms>=3.4,<3.7'],
        'cms': ['django-cms>=3.4,<3.7']
    },
    entry_points={
        'console_scripts': [
            'django-app-helper = app_helper.main:main',
        ]
    },
    test_suite='app_helper.tests',
    tests_require=test_requirements,
    classifiers=[
        'License :: OSI Approved :: GNU General Public License v2 or later (GPLv2+)',
        'Development Status :: 5 - Production/Stable',
        'Framework :: Django',
        'Framework :: Django :: 1.11',
        'Framework :: Django :: 2.0',
        'Framework :: Django :: 2.1',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
    ],
)
